--	LOOT SUMMARY - LUA
--
--	Author:			@Dysrel
--	File:			LootSummary.lua
--	Version:		Alpha 1.0
--	Date:			1-30-2014
--
--	Description:	Loot summary window for a quick glance
--					of looted items and Profit Loss for session
--
--	Usage Notes:	/ls 			- hide/unhide main window
--					/ls settings 	- hide/unhide settings
--					/ls clearlog 	- reset loot stats/log
--					/ls cleargold 	- reset gold stats
--					/ls memcheck 	- see addon memory usage
--					/ls memclear 	- optimize memory usage
--					/ls who 		- see author information
--					/ls help 		- see all slash commands
--
--	GLOBALS & LOCALS

--	table for holding loot event items from the API
local newLE 	= nil
--	table for passing certain event items from newLE
local LE 		= {}
--	table for building the UI and calculating
local LGin 		= nil
--	gold value total of looted items
local sTotal 	= 0
--	looted items quantity 
local sQty 		= 0
--	for holding information passed from GetItemLink()
local icon, sPrice, req, eType, iStyle
--	user manual hide selection
local userHidden	= false
--	show help on initial load
local showHelp		= true
--	show help time from initial load
local startupTS		= GetGameTimeMilliseconds()
--	Get Bag Slots - concept from BazgrimsToolbar Addon (Bazgrim)
local bagIcon, bagSlots	= GetBagInfo(BAG_BACKPACK)
local bagSpace			= 0

--	Create the LS (Loot Summary Object) for use throughout the addon
LS = { }

--	LS Version
LS.version	=	0.01

--	LS Defaults table for saving user variables
LS.defaults = 
{

	["ls_sX"]		= 0,
	["ls_sY"] 		= 0,
	["trackGold"]	= true,
	["logCL"]		= true,
	["startHidden"]	= false
	
}

--	LS Initialization table
LS.Init = {}

--	LE Init Table for holding Loot Event First Round Tables
LE.Init = {}

--	DEBUG: Set the debugging flag for use throughout the code
LS.debug = false

--	Function fired at addon loaded to setup variables and default settings
function LS.Initialize( self, addOnName )

	--	Check for addon loaded, then initialize, otherwise bail
	if ( addOnName ~= "LootSummary" ) then return end

	--	LS Character table to hold friendly character information
	LS.character =
	{

		["name"] 		= GetUnitName( 'player' ),
		["race"]		= GetUnitRace( 'player' ),
		["class"]		= GetUnitClass( 'player' ),
		["nicename"]	= string.gsub( GetUnitName( 'player' ) , "%-", "%%%-"),
		["exp"]			= GetUnitXP('player'),

	}
	
	--	LS Gold History for tracking Profit & Loss per gaming session or addon reload
	LS.goldhistory =
	{
		["iGold"] 		= 0,
		["oGold"]		= 0,
		["PL"]			= 0,
		["plStartTime"]	= GetGameTimeMilliseconds(),
		["GPM"]			= 0
	}

	--	Load saved variables
	LS.Vars = ZO_SavedVars:New( "lsVars" , math.floor( LS.version * 100 ) , nil , LS.defaults , nil )

	--	Register the LOOT Gained event for use in the addon
	--	usage (str receivedBy, str itemName, int quantity, int itemSound, int lootType, bool self)
	EVENT_MANAGER:RegisterForEvent( "LS" , EVENT_LOOT_RECEIVED, LS.LootGained)
	
	--	Register the MONEY Update event for use in the addon
	--	usage (int newMoney, int oldMoney, int reason)
	EVENT_MANAGER:RegisterForEvent( "LS" , EVENT_MONEY_UPDATE, LS.MoneyUpdate)

	--	Register the INVENTORY BAG Update event for use in the addon
	EVENT_MANAGER:RegisterForEvent("LS", EVENT_INVENTORY_SINGLE_SLOT_UPDATE, LS.BagUpdate)

	--	Future EVENTS to experiment with
	--	EVENT_FINESSE_RANK_CHANGED (string unitTag, luaindex rankNum, string name, integer xpBonus, bool loot)
	--	EVENT_LOOT_CLOSED
	--	EVENT_LOOT_ITEM_FAILED (integer reason, string itemName)
	--	EVENT_LOOT_UPDATED

	
	-- 	Set the default position of the summary window based on variable in the config file
	ls_s:SetAnchor(TOPLEFT,GuiRoot,TOPLEFT,LS.Vars.ls_SX,LS.Vars.ls_SY)
		
	-- 	Load default settings into the button control Option #1
	if ( LS.Vars.trackGold == true ) then
		ls_gset_b1:SetText('Enabled')
		ls_s_GoldT:SetText( "|cFFFF00 [ + Gold: |r|c00FF00 0|r |cFFFF00]  [ - Gold:|r|cFF0000 0|r |cFFFF00]  [ P/L: |c00FF00 0 |r |cFFFF00]  [ GPM: |c00FF00 0 |r |cFFFF00] " )
	else
		ls_gset_b1:SetState( BSTATE_NORMAL )
		ls_gset_b1:SetText('Disabled')
		ls_s_GoldT:SetText( "[ Not Tracking Gold ]" )
	end

	-- 	Load default settings into the button control Option #2
	if ( LS.Vars.logCL == true ) then
		ls_gset_b2:SetText('Enabled')
	else
		ls_gset_b2:SetState( BSTATE_NORMAL )
		ls_gset_b2:SetText('Disabled')
	end

	-- 	Load default settings into the button control Option #3
	if ( LS.Vars.startHidden == true ) then
		ls_gset_b3:SetText('Enabled')
		lsSum:SetHidden(true)
		d ("** Loot Summary System Hidden ** (type /ls to unhide)")
		userHidden = true
	else
		ls_gset_b3:SetState( BSTATE_NORMAL )
		ls_gset_b3:SetText('Disabled')
	end

	--	Update the bag information after the addon is loaded
	--	TODO: Update the inventory system to use colors like Bazgrim's Toolbar, after permission obtained
	LS.BagUpdate()
	--	Update the summary window to indicate any changes after load (primarily the inventory counter)
	ls_s_LootT:SetText( "|cFFFF00 [ Items Looted: |r|c00FF00"..sQty.."|r|cFFFF00 ]  [ Sell Value: |r|c00FF00"..sTotal.." |r|cFFFF00]  [ Inv: "..(string.format("|r|c00FF00%d / %d", bagSlots-bagSpace, bagSlots)).."|r|cFFFF00 ]")
	
	--	DEBUG: Memory tracking tag
	-- 	usage (int stat (1 Count, 2 Collect, 3 Restart 4 Stop))
	if LS.debug == true then
		LS.MemoryChk(1,"at addon load-time")
	end
end

--	Hook initialization onto the ADD_ON_LOADED event
EVENT_MANAGER:RegisterForEvent( "LS" , EVENT_ADD_ON_LOADED , LS.Initialize )

--	********************************************
--	START LOOT SUMMARY WINDOW MANAGEMENT
--	********************************************
	--	Loot Summary Top Level Control 
	ls_s = WINDOW_MANAGER:CreateTopLevelWindow("lsSum")
	ls_s:SetMouseEnabled(true)
	ls_s:SetMovable(true)
	ls_s:SetDimensions(425,245)
	ls_s:SetHandler( "OnMouseDoubleClick" , function(self) lssMouseDouble(self) end)
	ls_s:SetHandler( "OnMouseUp" , function(self) LS.S_MouseUp(self) end)
	ls_s:SetHandler( "OnMouseWheel" , 
	function(self,delta)
		ls_s_TB:SetScrollPosition(lsTB:GetScrollPosition() + delta)
	end)

	--	Loot Summary Control - Title1
	ls_s_Title = WINDOW_MANAGER:CreateControl("lsTitle",lsSum,CT_LABEL)
	ls_s_Title:SetFont( "ZoFontGame" )
	ls_s_Title:SetColor(0,255,255,1.5)
	ls_s_Title:SetText( "Loot Summary System" )
	ls_s_Title:SetAnchor(TOPLEFT,lsSum,TOPLEFT,10,0)

	--	Loot Summary Control - Gold Tracking Summary Title
	ls_s_GoldT = WINDOW_MANAGER:CreateControl("lsGoldT",lsSum,CT_LABEL)
	ls_s_GoldT:SetFont( "ZoFontGameBold" )
	ls_s_GoldT:SetColor(1,255,1,.7)
	ls_s_GoldT:SetText( "|cFFFF00 [ + Gold: |r|c00FF00 0|r |cFFFF00]  [ - Gold:|r|cFF0000 0|r |cFFFF00]  [ P/L: |c00FF00 0 |r |cFFFF00]  [ GPM: |c00FF00 0 |r |cFFFF00] " )
	ls_s_GoldT:SetAnchor(TOP,lsSum,TOP,0,19)

	--	Loot Summary Control - Loot Tracking Summary
	ls_s_LootT = WINDOW_MANAGER:CreateControl("lsLootT",lsSum,CT_LABEL)
	ls_s_LootT:SetFont( "ZoFontGameBold" )
	ls_s_LootT:SetColor(255,1,1,.7)
	ls_s_LootT:SetText( "|cFFFF00 [ Items Looted: |r|c00FF00"..sQty.."|r|cFFFF00 ]  [ Sell Value: |r|c00FF00"..sTotal.." |r|cFFFF00]  [ Inv: "..(string.format("|r|c00FF00%d / %d", bagSlots-bagSpace, bagSlots)).."|r|cFFFF00 ]")
	ls_s_LootT:SetAnchor(TOP,lsSum,TOP,0,38)

	--	Loot Summary Control - System Button [X]
	ls_s_hideBtn = WINDOW_MANAGER:CreateControl("lsHideBTN" , lsSum , CT_BUTTON)
	ls_s_hideBtn:SetDimensions( 35 , 25 )
	ls_s_hideBtn:SetFont("ZoFontGameBold")
	ls_s_hideBtn:SetAnchor(TOPRIGHT,lsSum,TOPRIGHT,0,0)
	ls_s_hideBtn:SetNormalFontColor(0,255,255,.7)
	ls_s_hideBtn:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_s_hideBtn:SetText('[X]')
	ls_s_hideBtn:SetState( BSTATE_NORMAL )
	ls_s_hideBtn:SetHandler( "OnClicked" , function(self) LS.LSSystemHide() end )

	--	Loot Summary Control - Settings Button [S]
	ls_s_setBtn = WINDOW_MANAGER:CreateControl("lsSetBTN" , lsSum , CT_BUTTON)
	ls_s_setBtn:SetDimensions( 35 , 25 )
	ls_s_setBtn:SetFont("ZoFontGameBold")
	ls_s_setBtn:SetAnchor(TOPRIGHT,lsSum,TOPRIGHT,-(lsHideBTN:GetWidth()),0)
	ls_s_setBtn:SetNormalFontColor(0,255,255,.7)
	ls_s_setBtn:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_s_setBtn:SetText('[ST]')
	ls_s_setBtn:SetState( BSTATE_NORMAL )
	ls_s_setBtn:SetHandler( "OnClicked" , function(self) LS.SettingsDisplay() end )

	--	Loot Summary Control - Clear Log Button [CL]
	ls_s_clrBtn = WINDOW_MANAGER:CreateControl("lsClrBTN" , lsSum , CT_BUTTON)
	ls_s_clrBtn:SetDimensions( 35 , 25 )
	ls_s_clrBtn:SetFont("ZoFontGameBold")
	ls_s_clrBtn:SetAnchor(TOPRIGHT,lsSum,TOPRIGHT,-(lsHideBTN:GetWidth() + lsSetBTN:GetWidth()),0)
	ls_s_clrBtn:SetNormalFontColor(0,255,255,.7)
	ls_s_clrBtn:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_s_clrBtn:SetText('[CL]')
	ls_s_clrBtn:SetState( BSTATE_NORMAL )
	ls_s_clrBtn:SetHandler( "OnClicked" , function(self) LS.ClearLog() end )

	--	Loot Summary Control - Clear Gold Stat Button [CG]
	ls_s_gldBtn = WINDOW_MANAGER:CreateControl("lsGldBTN" , lsSum , CT_BUTTON)
	ls_s_gldBtn:SetDimensions( 35 , 25 )
	ls_s_gldBtn:SetFont("ZoFontGameBold")
	ls_s_gldBtn:SetAnchor(TOPRIGHT,lsSum,TOPRIGHT,-(lsHideBTN:GetWidth() + lsSetBTN:GetWidth() + lsClrBTN:GetWidth()),0)
	ls_s_gldBtn:SetNormalFontColor(0,255,255,.7)
	ls_s_gldBtn:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_s_gldBtn:SetText('[CG]')
	ls_s_gldBtn:SetState( BSTATE_NORMAL )
	ls_s_gldBtn:SetHandler( "OnClicked" , function(self) LS.ClearGold() end )

	--	Loot Summary Control - Window Background
	ls_s_BG = WINDOW_MANAGER:CreateControl("lsSumBG",lsSum,CT_BACKDROP)
	ls_s_BG:SetDimensions( 425 , 245 )
	ls_s_BG:SetCenterColor(0.1,0.1,0.1,1)
	ls_s_BG:SetEdgeColor(0,0,0,1)
	ls_s_BG:SetEdgeTexture("", 8, 1, 1)
	ls_s_BG:SetAlpha(0.5)
	ls_s_BG:SetAnchor(BOTTOM,lsSum,BOTTOM,0,0)

	--	Loot Summary Control - Text Buffer Control
	ls_s_TB = WINDOW_MANAGER:CreateControl("lsTB",lsSum,CT_TEXTBUFFER)
	ls_s_TB:SetDimensions( 415, 175 )
	ls_s_TB:SetFont( "ZoFontGame" )
	ls_s_TB:SetAnchor(BOTTOM,lsSum,BOTTOM,0,-5)
	ls_s_TB:SetLinkEnabled( true )
	--	TODO: Add more code once the API supports links in the Textbuffer
	--	OnLinkClicked NOT WORKING IN TEXTBUFFER AS API INDICATED
	--	Inherits: OnSetControlEventHandler
	--	ScriptArguments: local self, linkData, linkText, button, ctrl, alt, shift, command = ...
	--	ls_s_TB:SetHandler( "OnLinkClicked" , 
	--	function(self,linkData,linkText)
	--		d ( tostring(self)..tostring(linkD).."**"..tostring(linkT) )
	--	end)

	--	Loot Summary Control - TextBuffer Background
	ls_s_TBBG = WINDOW_MANAGER:CreateControl("lsSumTBBG",lsSum,CT_BACKDROP)
	ls_s_TBBG:SetDimensions( 415 , 175 )
	ls_s_TBBG:SetCenterColor(0.1,0.1,0.1,1)
	ls_s_TBBG:SetEdgeColor(0,0,0,1)
	ls_s_TBBG:SetEdgeTexture("", 8, 1, 1)
	ls_s_TBBG:SetAlpha(0.8)
	ls_s_TBBG:SetAnchor(BOTTOM,lsTB,BOTTOM,0,0)

--	********************************************
--	END LOOT SUMMARY WINDOW MANAGEMENT
--	********************************************

--	********************************************
--	START ADDON SETTINGS WINDOW
--	********************************************
	--	Addon Settings Window 
	ls_set = WINDOW_MANAGER:CreateTopLevelWindow("lsSettings")
	ls_set:SetMouseEnabled(true)
	ls_set:SetHidden( true )
	ls_set:SetMovable(true)
	ls_set:SetDimensions(400,400)
	ls_set:SetAnchor(BOTTOM,GuiRoot,BOTTOM,0,-200)
	
	--	Addon Settings Window - Title
	ls_set_Title = WINDOW_MANAGER:CreateControl("lsSTitle",lsSettings,CT_LABEL)
	ls_set_Title:SetDimensions( ls_set:GetWidth() , 36 )
	ls_set_Title:SetFont( "ZoFontWindowTitle" )
	ls_set_Title:SetColor(1,1,1,1)
	ls_set_Title:SetHorizontalAlignment(1)
	ls_set_Title:SetVerticalAlignment(0)
	ls_set_Title:SetText( "Loot Summary Settings")
	ls_set_Title:SetAnchor(TOP,ls_set,TOP,0,10)

	--	Addon Settings Window - Close Button (X)
	ls_set_clsBtn = WINDOW_MANAGER:CreateControl("lsSClose" , lsSettings , CT_BUTTON)
	ls_set_clsBtn:SetDimensions( 25 , 25 )
	ls_set_clsBtn:SetFont("ZoFontGameBold")
	ls_set_clsBtn:SetAnchor(TOPRIGHT,lsSettings,TOPRIGHT,-5,5)
	ls_set_clsBtn:SetNormalFontColor(1,1,1,1)
	ls_set_clsBtn:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_set_clsBtn:SetText('[X]')
	ls_set_clsBtn:SetState( BSTATE_NORMAL )
	ls_set_clsBtn:SetHandler( "OnClicked" , function(self) LS.SettingsDisplay() end )

	--	Addon Settings Window - Close Button Label at base (Close)
	ls_set_clsBtn = WINDOW_MANAGER:CreateControl("lsSClose2" , lsSettings , CT_BUTTON)
	ls_set_clsBtn:SetDimensions( 175 , 26 )
	ls_set_clsBtn:SetFont("ZoFontGameBold")
	ls_set_clsBtn:SetAnchor(BOTTOM,lsSettings,BOTTOM,0,-10)
	ls_set_clsBtn:SetNormalFontColor(1,1,1,1)
	ls_set_clsBtn:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_set_clsBtn:SetText('[ Close Settings ]')
	ls_set_clsBtn:SetState( BSTATE_NORMAL )
	ls_set_clsBtn:SetHandler( "OnClicked" , function(self) LS.SettingsDisplay() end )

	--	Addon Settings Window - Note
	ls_set_note = WINDOW_MANAGER:CreateControl("lsSNote",lsSettings,CT_LABEL)
	ls_set_note:SetDimensions( ls_set:GetWidth() , 26 )
	ls_set_note:SetFont( "ZoFontGame" )
	ls_set_note:SetColor(1,1,1,1)
	ls_set_note:SetText( "*-requires restart" )
	ls_set_note:SetAnchor(BOTTOMLEFT,lsSettings,BOTTOMLEFT,10,-45)

	--	Addon Settings Window - Message
	ls_set_mess = WINDOW_MANAGER:CreateControl("lsSMess",lsSettings,CT_LABEL)
	ls_set_mess:SetDimensions( ls_set:GetWidth() , 20 )
	ls_set_mess:SetFont( "ZoFontGame" )
	ls_set_mess:SetColor(1,1,1,1)
	ls_set_mess:SetHorizontalAlignment(1)
	ls_set_mess:SetVerticalAlignment(0)
	ls_set_mess:SetText( "LS Version: "..math.floor( LS.version * 100 ))
	ls_set_mess:SetAnchor(TOP,lsSTitle,BOTTOM,0,0)

	--	Addon Settings Window - General Settings Control
	ls_gset = WINDOW_MANAGER:CreateControl("lsGeneralSettings",lsSettings,CT_CONTROL)
	ls_gset:SetDimensions( lsSettings:GetWidth() - 100 , 160 )
	ls_gset:SetAnchor(TOPLEFT,ls_set_mess,BOTTOMLEFT,50,10)
	
	--	Addon Settings Window - General Settings Heading
	ls_gset_head = WINDOW_MANAGER:CreateControl("lsGSHeading",lsGeneralSettings,CT_LABEL)
	ls_gset_head:SetDimensions( lsGeneralSettings:GetWidth() , 48 )
	ls_gset_head:SetFont("ZoFontHeader2")
	ls_gset_head:SetColor(1,1,1,1)
	ls_gset_head:SetVerticalAlignment(1)
	ls_gset_head:SetAnchor(TOP,lsGeneralSettings,TOP,0,-10)
	ls_gset_head:SetText("General Settings")

	--	Addon Settings Window - General Settings Label for Tracking Gold Option
	ls_gset_l1 = WINDOW_MANAGER:CreateControl("lsGSL1",lsGeneralSettings,CT_LABEL)
	ls_gset_l1:SetDimensions( lsGeneralSettings:GetWidth() * 0.6 , 30 )
	ls_gset_l1:SetText("* Show Gold Statistics")
	ls_gset_l1:SetFont("ZoFontGame")
	ls_gset_l1:SetColor(1,1,1,1)
	ls_gset_l1:SetVerticalAlignment(1)
	ls_gset_l1:SetAnchor(TOPLEFT,lsGSHeading,BOTTOMLEFT,0,0)
	
	--	Addon Settings Window - General Settings Button for Tracking Gold Option
	ls_gset_b1 = WINDOW_MANAGER:CreateControl( "trackGold" , lsGeneralSettings , CT_BUTTON )
	ls_gset_b1:SetDimensions( lsGeneralSettings:GetWidth() * 0.4 , 30 )
	ls_gset_b1:SetFont("ZoFontGame")
	ls_gset_b1:SetVerticalAlignment(1)
	ls_gset_b1:SetHorizontalAlignment(2)
	ls_gset_b1:SetAnchor(TOPRIGHT,lsGSHeading,BOTTOMRIGHT,0,0)
	ls_gset_b1:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_gset_b1:SetNormalFontColor(0.6,0,0,1)
	ls_gset_b1:SetPressedFontColor(0,0.6,0,1)
	ls_gset_b1:SetText('Disabled')
	ls_gset_b1:SetState( BSTATE_PRESSED )
	ls_gset_b1:SetHandler( "OnClicked" , function(self) o1toggle(self,"trackGold") end )
		
	--	Addon Settings Window - General Settings Label for Logging to chatlog for parsing programs
	ls_gset_l2 = WINDOW_MANAGER:CreateControl("lsGSL2",lsGeneralSettings,CT_LABEL)
	ls_gset_l2:SetDimensions( lsGeneralSettings:GetWidth() * 0.6 , 30 )
	ls_gset_l2:SetText("Log to Chatlog")
	ls_gset_l2:SetFont("ZoFontGame")
	ls_gset_l2:SetColor(1,1,1,1)
	ls_gset_l2:SetVerticalAlignment(1)
	ls_gset_l2:SetAnchor(TOPLEFT,lsGSHeading,BOTTOMLEFT,0,31)


	--	Addon Settings Window - General Settings Button for Logging to chatlog for parsing programs
	ls_gset_b2 = WINDOW_MANAGER:CreateControl( "trackPL" , lsGeneralSettings , CT_BUTTON )
	ls_gset_b2:SetDimensions( lsGeneralSettings:GetWidth() * 0.4 , 30 )
	ls_gset_b2:SetFont("ZoFontGame")
	ls_gset_b2:SetVerticalAlignment(1)
	ls_gset_b2:SetHorizontalAlignment(2)
	ls_gset_b2:SetAnchor(TOPRIGHT,lsGSHeading,BOTTOMRIGHT,0,31)
	ls_gset_b2:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_gset_b2:SetNormalFontColor(0.6,0,0,1)
	ls_gset_b2:SetPressedFontColor(0,0.6,0,1)
	ls_gset_b2:SetText('Disabled')
	ls_gset_b2:SetState( BSTATE_PRESSED )
	ls_gset_b2:SetHandler( "OnClicked" , function(self) o1toggle(self,"logCL") end )

	--	Addon Settings Window - General Settings Label for Starting Hidden
	ls_gset_l3 = WINDOW_MANAGER:CreateControl("lsGSL3",lsGeneralSettings,CT_LABEL)
	ls_gset_l3:SetDimensions( lsGeneralSettings:GetWidth() * 0.6 , 30 )
	ls_gset_l3:SetText("* Start Log Hidden")
	ls_gset_l3:SetFont("ZoFontGame")
	ls_gset_l3:SetColor(1,1,1,1)
	ls_gset_l3:SetVerticalAlignment(1)
	ls_gset_l3:SetAnchor(TOPLEFT,lsGSHeading,BOTTOMLEFT,0,62)

	--	Addon Settings Window - General Settings Button for Starting Hidden
	ls_gset_b3 = WINDOW_MANAGER:CreateControl( "startHidden" , lsGeneralSettings , CT_BUTTON )
	ls_gset_b3:SetDimensions( lsGeneralSettings:GetWidth() * 0.4 , 30 )
	ls_gset_b3:SetFont("ZoFontGame")
	ls_gset_b3:SetVerticalAlignment(1)
	ls_gset_b3:SetHorizontalAlignment(2)
	ls_gset_b3:SetAnchor(TOPRIGHT,lsGSHeading,BOTTOMRIGHT,0,62)
	ls_gset_b3:SetMouseOverFontColor(0.8,0.4,0,1)
	ls_gset_b3:SetNormalFontColor(0.6,0,0,1)
	ls_gset_b3:SetPressedFontColor(0,0.6,0,1)
	ls_gset_b3:SetText('Disabled')
	ls_gset_b3:SetState( BSTATE_PRESSED )
	ls_gset_b3:SetHandler( "OnClicked" , function(self) o1toggle(self,"startHidden") end )

	--	Addon Settings Window - Window Background
	ls_set_BG = WINDOW_MANAGER:CreateControl("lsSettingsBG",lsSettings,CT_BACKDROP)
	ls_set_BG:SetDimensions( ls_set:GetWidth() , ls_set:GetHeight() )
	ls_set_BG:SetCenterColor(0,0,0,0.5)
	ls_set_BG:SetEdgeColor(.1,.1,.1,1)
	ls_set_BG:SetEdgeTexture("",8,1,2)
	ls_set_BG:SetAnchor(CENTER,lsSettings,CENTER,0,0)
	
--	********************************************
--	END ADDON SETTINGS WINDOW
--	********************************************

--	Function to track Gold Updates when looting or selling
--	usage: (int reason, new gold total, old gold total)
--	fires from EVENT_MANAGER:RegisterForEvent( "LS" , EVENT_MONEY_UPDATE, LS.MoneyUpdate)
function LS.MoneyUpdate(whyMon, nMon, oMon)

	--	No reason to run if user is not tracking
	if LS.Vars.trackGold == true then
		--	This function would be so much cooler if the reason why looted did actually fire with it
		--	API is not producing the data correctly. Nevertheless it tracks your loss and gain in gold
		--	and keeps a profit loss statement with Gold Per Minute
		
		--	if new is greater than old amounts then update the log and stats
		if nMon > oMon then
			lsTB:AddMessage("|c00FF00 You Gained [|r"..(nMon-oMon).."|c00FF00] Gold!|r")
			if ( LS.Vars.logCL == true ) then
				d ( "-->> |c00FF00 You Gained [|r"..(nMon-oMon).."|c00FF00] Gold!|r" )
			end
			
		--	Populate the incoming gold table variable
			LS.goldhistory.iGold = LS.goldhistory.iGold + (nMon-oMon)
			
		else
			lsTB:AddMessage("|cFF0000 You Lost [|r"..(nMon-oMon).."|cFF0000] Gold!|r")
			if ( LS.Vars.logCL == true ) then
				d ( "-->> |cFF0000 You Lost [|r"..(nMon-oMon).."|cFF0000] Gold!|r" )
			end
			--	Populate the outgoing gold table variable
			LS.goldhistory.oGold = LS.goldhistory.oGold + (nMon-oMon)
		end
		
		--	Since we are carrying and using the negative number use the following PL Calc
		LS.goldhistory.PL = LS.goldhistory.iGold + LS.goldhistory.oGold
			
		--	Calculate the GPM for the diehard players that are tracking their gains and losses
		LS.goldhistory.GPM = string.format("%.2f", LS.goldhistory.PL / (ms2mins(GetGameTimeMilliseconds() - LS.goldhistory.plStartTime))) 
		
		--	Update the summary window with the stats
		ls_s_GoldT:SetText( "|cFFFF00 [ + Gold: |r|c00FF00"..LS.goldhistory.iGold.."|r |cFFFF00]  [ - Gold: |r|cFF0000"..LS.goldhistory.oGold.."|r |cFFFF00]  [ P/L: |c00FF00"..LS.goldhistory.PL.."|r |cFFFF00]  [ GPM: |c00FF00"..LS.goldhistory.GPM.."|r |cFFFF00] " )
		
	end

end

--	Function to track Loot Updates
--	usage: (str receivedBy, str itemName, int quantity, int itemSound, int lootType, bool self)
--	fires from EVENT_MANAGER:RegisterForEvent( "LS" , EVENT_LOOT_RECEIVED, LS.LootGained)
function LS.LootGained(eventCode, receivedBy, itemName, quantity, itemSound, lootType, self)

	--	Clear the round one event table
	LE.LGEvent = {}

	--	populate the running event table
	newLE = 
		{
			["by"]			=	receivedBy,
			["itemname"]	=	itemName,
			["qty"]			=	quantity,
			["itemsound"]	=	itemSound,
			["loottype"]	=	lootType,
			["selfbool"]	=	self
		}
	
	--	insert data into the round one event table
	table.insert( LE.LGEvent, newLE)

	--	Call the update function to calc stats and move out of the event
	--	Leaving the event allows us to take care of data without bothering another
	--	possible event
	LS.UpdateSummary(LE.LGEvent)
	
end

--	Function to control the stat updates and TextBuffer updates to the summary window
--	usage: (table LG_In) table containing looted item information
--	Fires from LS.LootGained()
function LS.UpdateSummary(LG_In)

	for k,v in pairs( LG_In ) do
		--	If looted by player then proceed
		if	(v.selfbool == true) then
			--	get item information
			--	GetItemLinkInfo Returns: string icon, integer sellPrice, bool meetsUsageRequirement, integer equipType, integer itemStyle
			icon, sPrice, req, eType, iStyle = GetItemLinkInfo(v.itemname)
			
			--	Full Message when Link Info gets the appropriate itemization values
			--	TODO: when link information is more verbose in the API update the stats shown
			--	lsTB:AddMessage( "Looted "..v.qty.." of "..v.itemname.." value: "..sPrice.." Ltype: "..v.loottype.." Etype: "..eType.." Estyle: "..iStyle)
		
			--	Strip junk characters from the link item
			--	TODO: Add more characters as discovered
			local lootNamenice = string.gsub(v.itemname, "(^p)", "")
			
			lootNamenice = string.gsub(lootNamenice, "(^n)", "")
			
			--	Get total sell value by multiplying quantity
			tPrice = v.qty * sPrice
			
			lsTB:AddMessage( "|cff8000 Looted  [|r"..v.qty.."|cff8000] |r"..lootNamenice.."|cff8000 uValue: |r"..sPrice.."|cff8000 qValue: |r"..tPrice)
			
			--	Log to chatlog if user has the setting enabled
			if ( LS.Vars.logCL == true ) then
				d ( "-->> |cff8000 Looted  [|r"..v.qty.."|cff8000] |r"..lootNamenice.."|cff8000 uValue: |r"..sPrice.."|cff8000 qValue: |r"..tPrice )
			end
			
			--	increment total sell value
			sTotal = sTotal + tPrice
			
			--	increment total quantity of items looted
			sQty = sQty + v.qty
			
			ls_s_LootT:SetText( "|cFFFF00 [ Items Looted: |r|c00FF00"..sQty.."|r|cFFFF00 ]  [ Sell Value: |r|c00FF00"..sTotal.." |r|cFFFF00]  [ Inv: "..(string.format("|r|c00FF00%d / %d", bagSlots-bagSpace, bagSlots)).."|r|cFFFF00 ]")
			
		return end
	end
end

--	Function save location of the summary Window after user moves it
--	usage: (control self)
--	fires from UI Control lsSum
function LS.S_MouseUp(self)
    -- assign the LEFT point of the window
    local left = self:GetLeft()
    -- assign the TOP point of the window
    local top = self:GetTop()
    -- Save the position information for LEFT and TOP to the variables file
    LS.Vars.ls_SX = left
    LS.Vars.ls_SY = top
end

--	Function to open or close settings window
--	usage: (control self)
--	Fires from UI Control lsSum, [ST] Button in Summary Window or /ls settings slash command
function lssMouseDouble(self)
	if (lsSettings:IsHidden() == true) then
		lsSettings:SetHidden(false)
	else
		lsSettings:SetHidden(true)
	end		
end

--	Function to toggle the settings buttons and state (universal)
--	usage: (int button, str "setting name")
--	Fires from Control functions for button clicks in settings window
--	Ignore than name o1 was for control 1 until I made it universal, too
--	lazy to fix it!
function o1toggle( button , component)

	local label = button:GetLabelControl()
	local status = true
	if ( label:GetText() == "Disabled" ) then
		button:SetState( BSTATE_PRESSED )
		label:SetText( "Enabled" )
	else
		button:SetState( BSTATE_NORMAL )
		label:SetText( "Disabled" )
		status = false
	end
	
	LS.Vars[component] = status
	
end

--	Function to generate hrs, mins, secs from millisecs returning mins in this case
--	usage: (int gametimemilliseconds)
--	Fires from random calls when needed
function ms2mins(mss)
	local milli = mss
	local secs = milli / 1000;
	local mins = secs / 60;
   	local hrs = mins / 60;
	return mins
end

--	Function to update bags on Slot Update
--	usage: ()
--	Fires from Addon Loaded Event and EVENT_INVENTORY_SINGLE_SLOT_UPDATE
function LS.BagUpdate()
	-- Cycle through bag slots --
	for i = bagSlots, 0, -1 do
		bagSpace = i
		if CheckInventorySpaceSilently(bagSpace) then break end
	end
end

--	Function to clear the TextBuffer log
--	usage: ()
--	Fires from user request
function LS.ClearLog()
	lsTB:Clear()
	LS.BagUpdate()
	sTotal = 0
	sQty = 0
	ls_s_LootT:SetText( "|cFFFF00 [ Items Looted: |r|c00FF00"..sQty.."|r|cFFFF00 ]  [ Sell Value: |r|c00FF00"..sTotal.." |r|cFFFF00]  [ Inv: "..(string.format("|r|c00FF00%d / %d", bagSlots-bagSpace, bagSlots)).."|r|cFFFF00 ]")
	d ("** Loot Summary Log and Stats Cleared **")
end

--	Function to Hide or Display the Settings Window
--	usage: ()
--	Fires from multiple user requests
function LS.SettingsDisplay()
	if (lsSettings:IsHidden() == true) then
		lsSettings:SetHidden(false)
	else
		lsSettings:SetHidden(true)
	end		
end

--	Function to clear Gold Statistics
--	usage: ()
--	Fires from multiple user requests
function LS.ClearGold()

	if ( LS.Vars.trackGold == true ) then
	-- LS Gold History for tracking Profit & Loss per gaming session or addon reload
		LS.goldhistory =
		{
			["iGold"] 		= 0,
			["oGold"]		= 0,
			["PL"]			= 0,
			["plStartTime"]	= GetGameTimeMilliseconds(),
			["GPM"]			= 0
		}
		-- Update the summary window with the stats
		ls_s_GoldT:SetText( "|cFFFF00 [ + Gold: |r|c00FF00"..LS.goldhistory.iGold.."|r |cFFFF00]  [ - Gold: |r|cFF0000"..LS.goldhistory.oGold.."|r |cFFFF00]  [ P/L: |c00FF00"..LS.goldhistory.PL.."|r |cFFFF00]  [ GPM: |c00FF00"..LS.goldhistory.GPM.."|r |cFFFF00] " )
		d ("** Gold Tracking History Reset **")
	else
	-- LS Gold History for tracking Profit & Loss per gaming session or addon reload
		LS.goldhistory =
		{
			["iGold"] 		= 0,
			["oGold"]		= 0,
			["PL"]			= 0,
			["plStartTime"]	= GetGameTimeMilliseconds(),
			["GPM"]			= 0
		}
		-- Update the summary window with the stats
		ls_s_GoldT:SetText( "[ Not Tracking Gold ]" )
		d ("** Gold Tracking History Reset **")
	end

end

--	Function to hide the main summary window (control)
--	usage: ()
--	Fires from multiple user requests
function LS.LSSystemHide()
	if (lsSum:IsHidden() == true) then
		lsSum:SetHidden(false)
		d ("** Loot Summary System Displayed **")
		--	set the userhidden flag to handle the OnUpdate issues
		userHidden = false
	else
		lsSum:SetHidden(true)
		d ("** Loot Summary System Hidden ** (type /ls to unhide)")
		--	set the userhidden flag to handle the OnUpdate issues
		userHidden = true
	end		
end

--	Function to handle user slash commands
--	usage (str text)
--	Fires from SLASH_COMMANDS["/ls"] = commandHandler
function commandHandler( text )
	if text=="" then 
		LS.LSSystemHide()
	elseif text=="settings" then
		LS.SettingsDisplay()
	elseif text=="clearlog" then
		LS.ClearLog()
	elseif text=="cleargold" then
		LS.ClearGold()
	elseif text=="help" then
		d ( "---> |cFFFF00 Loot Summary Version |c00FF00"..LS.version.."|r Help  <---" )
		d ( "( Type, |c00FF00/ls|r ) to hide/unhide main window" )
		d ( "( Type, |c00FF00/ls settings|r ) to hide/unhide settings" )
		d ( "( Type, |c00FF00/ls clearlog|r ) to reset loot stats/log" )
		d ( "( Type, |c00FF00/ls cleargold|r ) to reset gold stats" )
		d ( "( Type, |c00FF00/ls memcheck|r ) to see addon memory usage" )
		d ( "( Type, |c00FF00/ls memclear|r ) to optimize addon memory usage" )
		d ( "( Type, |c00FF00/ls who|r ) to view author contact info" )
	elseif text=="who" then
		d ( "---> |cFFFF00 Loot Summary Addon by <---" )
		d ( "---> |cFFFF00 Author: @Dysrel |r    <---" )

	elseif text=="memcheck" then
		LS.MemoryChk(1,"User Requested Memory Check")
	elseif text=="memclear" then
		LS.MemoryChk(2,"User Memory Optimize, is Finished")
	elseif text=="who" then
		d ( "( Author: @Dysrel in-game and on forums )" )
	end
end

-- Slash Commands --
SLASH_COMMANDS["/ls"] = commandHandler

--	function to track and or optimize memory
--	usage (int stat (1 Count, 2 Collect))
--	Fires from slash commands and debugger
function LS.MemoryChk(stat, msg)

	if stat == 1 then
		-- Count Only
		local memorys = collectgarbage("count")
		memorys = memorys / 1024
		lsTB:AddMessage(string.format("Using %.2f" , memorys) .. " MB - " .. msg)
		if ( LS.Vars.logCL == true ) then
			d (string.format("Using  %.2f" , memorys) .. " MB - " .. msg)
		end
	elseif stat == 2 then
		-- Collect Garbage
		local memorys = collectgarbage()
		lsTB:AddMessage( msg )
		if ( LS.Vars.logCL == true ) then
			d ( msg )
		end
		
	end
	
end
	

--	Function runs on every frame update (Very Fast)
--	usage: ()
--	Fires from XML Control lsBLANK
function LS.Update(self)
	
	--	Just a quick built in delay so chat window updates after load
	--	This code fires once per load
	if (showHelp == true) and ( (GetGameTimeMilliseconds() - startupTS) > 5000 )then
		d ("--> Loot Summary Loaded Version: "..LS.version)
		if (lsSum:IsHidden() == true) then
			d ("---> Summary is hidden by default, in settings!")
		end		
		d ("---> ( Type, /ls ) to hide/unhide main window")
		d ("---> ( Type, /ls help ) for all slash commands")
		showHelp = false
	end

	-- is menu hidden?
	menuHidden = ZO_KeybindStripControlBackground:IsHidden()
	-- is interact window hidden?
	interactHidden = ZO_InteractWindow:IsHidden()
	-- Is user hiding the window?
	userHidden = userHidden
		
	-- show LS System if user is not using menus or windows
	if (menuHidden == true) and (interactHidden == true) and (userHidden == false) then
		lsSum:SetHidden(false)

	elseif (menuHidden == false) or (interactHidden == false) or (userHidden == true) then
		lsSum:SetHidden(true)
	end

end